import numpy as np
import soundfile as sf
from scipy.linalg import toeplitz
import matplotlib.pyplot as plt

# Read in audiofiles
speech, samplerate = sf.read('speech.wav')
noise, samplerate = sf.read('car_noise.wav')

# Create section of observed signal
n = 500
a = 0.1
y = speech[0:n] + a*noise[0:n]

# Auto-correlation matrix
ycorr_long = np.convolve(y,np.flip(y),mode='full')
ycorr = ycorr_long[n-1:]
R = toeplitz(ycorr)

# Cross-correlation vector
ys_corr_long = np.convolve(y,np.flip(speech[0:n]),mode='full')
ys_corr = ys_corr_long[n-1:]

# Optimal filter coefficients
hopt = np.matmul(np.linalg.inv(R),ys_corr)

# Apply filter
s_hat_long = np.convolve(y,hopt,mode='full')
s_hat = s_hat_long[0:n]

# Plot signals
plt.plot(speech[0:n])
plt.plot(y[0:n])
plt.plot(s_hat)
plt.legend((['speech','observed','estimated']))
plt.show(block='True')
